#
# Copyright (c) 1998 Regents of the University of California.
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions
# are met:
# 1. Redistributions of source code must retain the above copyright
#    notice, this list of conditions and the following disclaimer.
# 2. Redistributions in binary form must reproduce the above copyright
#    notice, this list of conditions and the following disclaimer in the
#    documentation and/or other materials provided with the distribution.
# 3. All advertising materials mentioning features or use of this software
#    must display the following acknowledgement:
#       This product includes software developed by the Computer Systems
#       Engineering Group at Lawrence Berkeley Laboratory.
# 4. Neither the name of the University nor of the Laboratory may be used
#    to endorse or promote products derived from this software without
#    specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
# ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
# ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
# FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
# DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
# OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
# HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
# LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
# OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
# SUCH DAMAGE.
#
# ------------
#
# Filename: app-caster.tcl
#   -- Created on Fri Aug 14 1998
#   -- Author: Yatin Chawathe <yatin@cs.berkeley.edu>
#
# Description:
#
#
#  @(#) $Header: /usr/src/mash/repository/mash/mash-1/tcl/applications/mashcast/app-mashcast.tcl,v 1.1 1998/08/14 22:19:48 yatin Exp $
#


import Application MashCast SRMv2_CacheProxy RemoteController


Class Application/MashCast -superclass Application



Application/MashCast public init {path argv} {
	$self next mashcast
	$self instvar proxy_ mc_
	
	set o [$self options]
	$self init_args $o
	$o parse_args $argv
	$o load_preferences "mashcast"

	puts "makeproxy [$self get_option makeProxy]"
	puts "noremotectrl [$self get_option noRemoteCtrl]"
	puts "porxyport [$self get_option srmv2CachePort]"
	puts "remotectrl [$self get_option remoteControllerPort]"
	if {[$self get_option makeProxy]==1} {
		puts "creating proxy"
		set proxy_ [new SRMv2_CacheProxy]
		$proxy_ open [$self get_option srmv2CachePort]
	}

	if {[$self get_option mungeURLs]==1} {
		set mc_ [new MashCast $path [$self get_option \
				srmv2CachePort]]
	} else {
		set mc_ [new MashCast $path]
	}

	if {[$self get_option noRemoteCtrl]!=1} {
		$mc_ remote_controller [$self get_option remoteControllerPort]
	}
}


Application/MashCast public destroy { } {
	$self instvar mc_ proxy_
	delete $mc_
	if [info exists proxy_] {delete $proxy_}
}


Application/MashCast public mashcast { } {
	return [$self set mc_]
}


Application/MashCast private init_args o {
	$o register_boolean_option -makeproxy makeProxy 1
	$o register_boolean_option -mungeurls mungeURLs 1
	$o register_boolean_option -noremotectrl noRemoteCtrl 1

	$o register_option -proxyport srmv2CachePort
	$o register_option -remotectrl remoteControllerPort

	$o add_default srmv2CachePort [SRMv2_CacheProxy get_option \
			srmv2CachePort]
	$o add_default remoteControllerPort [RemoteController get_option \
			remoteControllerPort]
}
