#
# Copyright (c) 1996 Regents of the University of California.
# All rights reserved.
# 
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions
# are met:
# 1. Redistributions of source code must retain the above copyright
#    notice, this list of conditions and the following disclaimer.
# 2. Redistributions in binary form must reproduce the above copyright
#    notice, this list of conditions and the following disclaimer in the
#    documentation and/or other materials provided with the distribution.
# 3. All advertising materials mentioning features or use of this software
#    must display the following acknowledgement:
#       This product includes software developed by the MASH Research
#       Group at the University of California Berkeley.
# 4. Neither the name of the University nor of the Research Group may be
#    used to endorse or promote products derived from this software without
#    specific prior written permission.
# 
# THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
# ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
# ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
# FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
# DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
# OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
# HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
# LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
# OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
# SUCH DAMAGE.
#
# @(#) $Header: /usr/src/mash/repository/mash/mash-1/tcl/applications/uc/ui-univ.tcl,v 1.4 1998/05/21 02:21:09 hodes Exp $
#

# import some default client UIs
import AVSwitchUI CameraUI PowerSwitchUI RvicClientUI

#
# a UI that wraps some "arbitrary" UI agents.
#
Class UniversalUI

# init method takes
# args are a tk window to parent the UniversalUI
# and a list of pairs of the form 
# <br> 'UI-agent-class' 'address-spec'
# <br>i.e.,<br> AVSwitchUI spade/1211 CameraUI spade/1212
#
UniversalUI public init {w args} {
    $self instvar components_ active_

    foreach {agent addrspec} $args {
	import $agent
	
	# if addrspec includes a name, turn it into IP addr
	set firstchar [string index $addrspec 0]
	if [string match \[a-zA-Z\] $firstchar] {
	    set n [lindex [split $addrspec "/"] 0]
	    set p [lindex [split $addrspec "/"] 1]
	    set s [gethostbyname $n]
	    if { $s == "" } {
		puts "cannot find address for '$n'... ignoring it."
		continue
	    }
	    set addrspec $s/$p
	}
	
	lappend components_($agent) $addrspec
	
    }
    
    # checkbuttons default off
    foreach i "[array names components_]" {
	set active_($i) 0
    }
    
    $self build_gui $w
    #after 3000 "$self refresh_ui"
}

UniversalUI private build_gui {w} {
    $self instvar components_ frames_ curAddr_ w_
    set w_ $w
    
    frame $w.listFrame -relief groove
    pack $w.listFrame -side left -fill y -expand 1
    
    # for each type we have a checkbox
    foreach uiType "[array names components_]" {
	checkbutton $w.listFrame.cb$uiType -text "$uiType" \
		-command "$self checkButtonInvoke $uiType"
	pack $w.listFrame.cb$uiType -side top -anchor w
	
	# under each type we have a radiobutton for each addr 
	set count 0
	foreach addr $components_($uiType) {
	    radiobutton $w.listFrame.rb$uiType$count -padx 10 -text "$addr" \
		    -command "$self radioButtonInvoke $uiType $addr" \
		    -variable dummyVar$uiType -value $count
	    pack $w.listFrame.rb$uiType$count -fill x -side top
	    if {$count == 0} {
		set curAddr_($uiType) $addr
		$w.listFrame.rb$uiType$count invoke
	    }
	    incr count
	}
	
    }
    
    # application UI side frame
    frame $w.appFrame 
    pack $w.appFrame -side right -fill y -expand 1
}

#
# toggles active_($agent), which indicates whether the UI for
# the agent is being shown
#
UniversalUI private checkButtonInvoke {agent} {
    #puts $agent

    $self instvar active_ agents_ w_ curAddr_
    
    if {$active_($agent) == 0} {
	# was off: build and pack it		
	frame $w_.appFrame.ag$agent
	pack $w_.appFrame.ag$agent
	set agents_($agent) \
		[new $agent $w_.appFrame.ag$agent $curAddr_($agent)]
	set active_($agent) 1
    } else {
	# was on, destroy the agent
	$agents_($agent) destroy
	pack forget $w_.appFrame.ag$agent
	destroy $w_.appFrame.ag$agent
	
	set active_($agent) 0
    }
    
}

UniversalUI private radioButtonInvoke {agent addr} {
    #puts "$agent $addr"
    $self instvar curAddr_ active_
    
    if {$curAddr_($agent) == $addr} {
	return
    }
    set curAddr_($agent) $addr
    if {$active_($agent)} {
	update
	$self checkButtonInvoke $agent
	update
	$self checkButtonInvoke $agent
	update
    }
}

UniversalUI private instantiateUI {agent} {
    if {[catch {new $agent .f$count $addrspec} retVal]} {
	puts "bad args: '$agent $addrspec'"
    } else {
	pack .f$count
	lappend components_ $retVal
	incr count
    }
}

# build_gui
# print_status
# repaint


