
#provide UDPChannel
import Network

# Open a new UDP channel.  If <i>spec</i> is given (in the
# form <i>address</i>/<i>port</i>), that address is used for
# sending and receiving all data.  If <i>spec</i> is not
# given, the channel is bound to an ephemeral port.
UDPChannel public init { {spec ""} {mtu 1024} } {
	$self next $mtu

	$self instvar net_
	set net_ [new Network]
	$net_ loopback 1

	if {$spec == ""} {
		$net_ open 0
	} else {
		set spec [split $spec /]
		set len [llength $spec]
		if { $len<2 || $len>3 } {
			$self fatal "invalid spec \"$spec\"."
		}
		set addr [lindex $spec 0]
		set ports [split [lindex $spec 1] :]

		#XXX
		set ttl 15
		if {$len == 3} {
			set ttl [lindex $spec 2]
		}

		if {[llength $ports] == 1} {
			set port [lindex $ports 0]
			$net_ open $addr $port $ttl
		} else {
			set sport [lindex $ports 0]
			set rport [lindex $ports 1]
			$net_ open $addr $sport $rport $ttl
		}
	}

	$self network $net_
}

#
UDPChannel public destroy {} {
	$self instvar net_
	delete $net_

	$self next
}

# Sets the multicast TTL for all outgoing packets to <i>t</i>.
UDPChannel public ttl {t} {
	$self instvar net_
	$net_ ttl $t
}
