#
# Copyright (c) 1998 Regents of the University of California.
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions
# are met:
# 1. Redistributions of source code must retain the above copyright
#    notice, this list of conditions and the following disclaimer.
# 2. Redistributions in binary form must reproduce the above copyright
#    notice, this list of conditions and the following disclaimer in the
#    documentation and/or other materials provided with the distribution.
# 3. All advertising materials mentioning features or use of this software
#    must display the following acknowledgement:
#       This product includes software developed by the Computer Systems
#       Engineering Group at Lawrence Berkeley Laboratory.
# 4. Neither the name of the University nor of the Laboratory may be used
#    to endorse or promote products derived from this software without
#    specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
# ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
# ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
# FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
# DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
# OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
# HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
# LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
# OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
# SUCH DAMAGE.
#
# ------------
#
# Filename: server-ssap.tcl
#   -- Created on Wed Mar  4 1998
#   -- Author: Angela Marie Schuett <schuett@cs.berkeley.edu>
#
# Description:
#
#
#  @(#) $Header: /usr/src/mash/repository/mash/mash-1/tcl/ssac/server-ssac.tcl,v 1.10 1998/08/05 22:41:06 schuett Exp $
#

import SessionCatalog ArchiveSession/Play/RTP ArchiveSession/Play/Mediaboard ArchiveSystem/Play

# This class handles parsing, operation of SSAC archive control protocol
# Need to clean this up, use Archive System instead
# <p> Status: pre-alpha
# <p> Author: Angela Schuett

Class SSACServer


# When this servent is created, the first msg is passed here.  All
# future messages arrive through recv_msg
SSACServer public init {app msg} {
	$self instvar app_ scheduled_ session_

	set app_ $app
	set scheduled_ ""
	set session_(seqno) 0
	
	$self parse $msg
	$self begin
	$self init_net
	#$self respond
	
	#$self start_time
	
	
}

# The SSAC Server receives a message if the sequence number has been
# incremented, otherwise it just acts as a keep-alive message and
# doesn't reach this level
SSACServer public recv_msg {msg} {
	
	$self instvar session_  app_

	
	
	$self parse $msg

	#for now, only implement time change
	$self change_time $session_(offset)
	$app_ set_seqno [incr session_(seqno)]
	
	
	
}



# parse the SSD field of AS1, which in this application, carries the
# SSAC protocol
SSACServer private parse {msg} {
	$self instvar sesslist_ session_
	
	
	puts "MSG: $msg :MSG"
	set lines [split $msg "\n"]

	set seqno [lindex [split [lindex $lines 0]] 1]

	if {$seqno > $session_(seqno) } {
	set session_(seqno) $seqno
	set session_(filename) [lindex [split [lindex $lines 1]] 1]
	set session_(offset) [lindex [split [lindex $lines 2]] 1]

		
		
	}

	
}

SSACServer private init_net {} {
	$self instvar sesslist_ session_ app_

	$app_ set_filename $session_(filename)
	$app_ set_duration $session_(duration)
	
	foreach sess $sesslist_ {
		$app_ add_sessinfo $sess $session_($sess,Address)
	}
	#puts "init seqno $session_(seqno)"
	$app_ set_seqno [incr session_(seqno)]
	
	
}


# To begin playback, we choose addresses, and create a player
# Right now, the unique session name is a critical piece of info that needs
# to be shared between client and server.  Is this the way we want it?
SSACServer private begin {} {
	$self instvar  session_ sesslist_ player_

	set player_ [new ArchiveSystem/Play]

	
	$player_ open $session_(filename)
	
	set sesslist_ [$player_ query_sessions]
	puts "Sesslist: $sesslist_"
	foreach sess $sesslist_ {
		set session_($sess,Address) [$self alloc_mcast_addr]/[$self alloc_port]
		# XXX hack
		set session_($sess,Media) [string trimright $sess "0123456789"]
		if {$session_($sess,Media) == "audio"} {
			set session_($sess,Address) $session_($sess,Address)/PCM/15
		} elseif {$session_($sess,Media) == "video"} {
			set session_($sess,Address) $session_($sess,Address)/null/15
		} else {
			set session_($sess,Address) $session_($sess,Address)/15
		}
		puts "********* $sess $session_($sess,Address) $session_($sess,Media) **********"
		$player_ create_playback_session $session_($sess,Address) $session_($sess,Media) $sess
	}
	
	

	$self change_time $session_(offset)
	

	set start [$player_ get_start]
	set end [$player_ get_end]
	set session_(duration) [expr $end - $start]
	puts $session_(duration)
	
}

# This is for demo purposes, or when you want to play from a set address
SSACServer private begin2 {} {
	$self instvar  session_ sesslist_ player_

	set player_ [new ArchiveSystem/Play]

	
	$player_ open $session_(filename)
	
	set sesslist_ [$player_ query_sessions]
	puts "Sesslist: $sesslist_"
	foreach sess $sesslist_ {
		
		set session_($sess,Media) [string trimright $sess "0123456789"]
		switch $session_($sess,Media) {
			video {
				set session_($sess,Address) 224.8.8.1/8000/null/15
			}
			audio {
				set session_($sess,Address) 224.8.8.2/8000/PCM/15
			}
			mediaboard {
				set session_($sess,Address) 224.8.8.3/8000/null/15
			}
		}

		
		puts "********* $sess $session_($sess,Address) $session_($sess,Media) **********"
		$player_ create_playback_session $session_($sess,Address) $session_($sess,Media) $sess
	}
	
	

	$self change_time $session_(offset)
	

	set start [$player_ get_start]
	set end [$player_ get_end]
	set session_(duration) [expr $end - $start]
	puts $session_(duration)
	
}

SSACServer public get_mapping {} {
	$self instvar player_

	return [$player_ get_mapping]

}



SSACServer private change_time {new} {
	$self instvar player_

	puts "NEW $new"
	if { [string compare $new "PAUSE"] == 0} {
		$player_ stop
	} else {
		$player_ start
		$player_ goto $new
		$player_ start
	}
	
}


#remove
SSACServer private start_time2 {} {
	$self instvar lts_ timemap_

	set newwords [split [lindex $timemap_ 1] ":"]
	set newwall [lindex $newwords 0]
	puts $newwall
	set newref [lindex $newwords 1]
	puts $newref

	$lts_ speed 0.0
	if {$newwall=="now"} {
		puts "logically [$lts_ now_logical]"
		puts "setting to $newref"
		$lts_ now_logical $newref
		set now [$lts_ now_system]
		$lts_ speed 1.0
		set timemap_ $now:$newref
		puts "Playing"
		
	} else {
		set nowwall [$lts_ now_system]
		puts "nowwall $nowwall"
		if {$newwall < $nowwall} {
			set diff [expr $nowwall - $newwall]
			puts "difference $diff"
			set newerref [expr $newref + $diff]
			puts "newerref $newerref"
			$lts_ now_logical $newerref
			$lts_ speed 1.0
			puts "LTS done setting speed"
			#keep timemap the same, for easy comparison
		} else {

			puts "XXX"
			$lts_ set_reference $newref $newwall
			# ???
			$lts_ speed 1.0
		}
	}
	

}


# XXX: Address allocation
# What happens if the port gets allocated to another 
# process in the meanwhile? 
# What happens if two different servers re-use the 
# same multicast address but have overlapped regions? 
# XXX need to follow sdp rules, actually, should be able to call some sdp proc # to do this



SSACServer instproc alloc_port {  } {
	# Random number U[8192, 16384]
	set r01 [expr [random]/double(0x7fffffff)]
	return [expr round(8392 + $r01 * 8192)]
}


SSACServer instproc alloc_mcast_addr {  } {
	set lo1 round([expr [random]/double(0x7fffffff) * 256])
	set lo2 round([expr [random]/double(0x7fffffff) * 256])
	return "224.2.[expr round($lo1)].[expr round($lo2)]"
}

