#
# Copyright (c) 1993-1995 Regents of the University of California.
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions
# are met:
# 1. Redistributions of source code must retain the above copyright
#    notice, this list of conditions and the following disclaimer.
# 2. Redistributions in binary form must reproduce the above copyright
#    notice, this list of conditions and the following disclaimer in the
#    documentation and/or other materials provided with the distribution.
# 3. All advertising materials mentioning features or use of this software
#    must display the following acknowledgement:
#	This product includes software developed by the Network Research
#	Group at Lawrence Berkeley National Laboratory.
# 4. Neither the name of the University nor of the Laboratory may be used
#    to endorse or promote products derived from this software without
#    specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
# ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
# ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
# FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
# DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
# OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
# HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
# LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
# OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
# SUCH DAMAGE.
#
# @(#) $Header: /usr/src/mash/repository/mash/mash-1/tcl/vic/ui-main.tcl,v 1.74 1998/10/09 23:39:55 yatin Exp $ (LBL)
#

import ControlMenu VicHelpWindow HelpWindow Switcher \
	ActiveSourceManager VicAudioUI AudioPanel UISrcList \
	CuesReceiver Configuration VisualFrame mashutils

#
# The main user interface for the vic application. 
# <p>
# An interface to video streams... <br>
# Displays a panel of postage-stamp-sized video sources and their respective network statistics.
# By selecting a thumbnail, a resizable toplevel window will appear that can remain dedicated to
# displaying the selected source or can be configured to cycle through all the video sources
# in a designated manner (voice-switched, timer-switched, etc.).
#
# <p>
# Defaults:
# <br> geometry used to be 250x225
# <br> vain can be set to true to allow voice-switch to switch to self
Class VicUI -superclass Observer -configuration {
	minwidth 200
	minheight 100
	filterGain 0.25
	geometry {}
	vain false
}

#
# Takes a frame, <i>w</i>, and create and pack within it the VIC title
# and 3 buttons: <br>
#    <dd> toggle Menu (<i>controlWindow</i>)
#    <dd> toggle Help (<i>helpWindow</i>)
#    <dd> Quit (<i>exitCmd</i>)
# <br>
# The caller is responsible for instantiating and packing <i>w</i>.
#
VicUI instproc build_menubar { w controlWindow helpWindow exitCmd } {
	frame $w -relief ridge -borderwidth 2
	label $w.title -text "[$self get_option appname] v[version]" -font [$self get_option smallfont] \
		-relief flat -justify left
	button $w.quit -text Quit -relief raised \
		-font [$self get_option smallfont] -command $exitCmd \
		-highlightthickness 1
	button $w.menu -text Menu -relief raised \
		-font [$self get_option smallfont] -highlightthickness 1 \
		-command "$controlWindow toggle"
	button $w.help -text Help -relief raised \
		-font [$self get_option smallfont] -highlightthickness 1 \
		-command "$helpWindow toggle"

	pack $w.title -side left -fill both -expand 1
	pack $w.menu $w.help $w.quit -side left -padx 1 -pady 1
}

#
# Set the title and iconname of the window "." to <i>prefix name</i>.
#
VicUI instproc window-title { prefix name } {
	$self instvar name_ prefix_
	set name_ $name
	set prefix_ $prefix

	wm iconname . "$prefix_$name_"
	wm title . "$prefix_$name_"

	#XXX
	proc mark_icon mark "$self mark-icon \$mark"
}

#
VicUI instproc mark-icon mark {
	$self instvar name_ prefix_
	global current_icon_mark
	if {$mark != $current_icon_mark} {
		set current_icon_mark $mark
		append mark $prefix_$name_
		wm iconname . $mark
	}
}

#
# Build the user-interface.
# XXX Get rid of this "spec" argument...
VicUI instproc init { w localChannel globalChannel videoAgent vpipe exitCmd {vspec {}} {CINDY_HACK_HAVE_VIDEO 1} } {
	$self next 

	$self instvar localChannel_ globalChannel_ videoAgent_ vpipe_ exitCmd_ 
	set localChannel_ $localChannel
	set globalChannel_ $globalChannel
	set videoAgent_ $videoAgent 
	set vpipe_ $vpipe
	set exitCmd_ "$exitCmd"
	
	# XXX this is a temporary hack while cindy works on decomposing VicUI so that MuiUI doesn't subclass from it... 
	# remove from args above and "$self next" call of MuiU::init
	if !$CINDY_HACK_HAVE_VIDEO {
		$self layout_gui $w	
		set conf [$self get_option conferenceName]
	} else {
		# let this ui be an observer of the VideoAgent so it can catch "trigger_sdes" 
		$videoAgent attach $self
		
		$self build_gui $w 
		
		if { $vspec == "" } {
			set conf "Contacting MeGa..."
		} else {
			set conf [$self get_option conferenceName]
		}
	}

	# set the window & icon title from the conference name
	set prefix [$self get_option iconPrefix]
	$self window-title $prefix $conf

	$self set-geometry

	if [$self yesno transmitOnStartup] {
		#XXX
		[$self set controlMenu_] build_window
		[$self set controlMenu_] invoke_transmit
	}

	$self init_keybindings "$exitCmd_"	
}

#
VicUI instproc reset {} {
	$self new_hostspec

	set conf [$self get_option conferenceName]
	$self window-title [$self get_option iconPrefix] $conf	
}

#
# Initialize some shortcut keys.
#
VicUI public init_keybindings { exitCmd } {
	#
	# emulate implicit keyboard focus
	#
	bind . <Enter> { focus %W }
	#wm focusmodel . active
	bind . <q> "$exitCmd"
	bind . <Control-c> "$exitCmd"
	bind . <Control-d> "$exitCmd"
}

#
# Given widget <i>w</i>, create frame $w.top, initializing its visual and colormap.  
# Within this frame, a frame ($w.top.f) is created for holding either a label (instvar label_)
# or another frame (instvar grid_).
# A few keybinbdings are assigned.
#
VicUI instproc build_gui { w } {
	$self instvar videoAgent_ userwindows_ path_ vpipe_ asm_ localChannel_ globalChannel_

	$self set_rate_vars [$videoAgent_ set session_]

	$self instvar id_
	set id_ [after 1000 "$self periodic_update"]

	#
	# Configure the Switcher class to ignore attempts to switch
	# to the local source, since this is not interesting (unless
	# we set the "vain" configuration option to true).
	#
	if { ![$self yesno vain] && [$videoAgent_ have_network] } {
		Switcher set ignore_([$videoAgent_ local]) 1
	}

	$self layout_gui $w	

	#
	# Local coordination bus events
	# - receive a focus speaker message from vat running on the
	#   same machine as this vic
	#
	set cb $localChannel_
	if { $cb != "" } {
		$cb register FOCUS_SPEAKER "$asm_ focus_speaker"
	}
	
	#
	# Global coordination bus events
	# - receive a aware {ear,hand,yes,no} message from vat
	#   running on others machine
	# - receive an unaware of the same types
	#
	# CuesReceiver handle these events, this is created only
	# when the user specified to do so on the command line
	#
	set cb $globalChannel_
	if { [$self yesno useCues] && $cb!="" }  {
		CuesReceiver set_cb $cb
	}
}

#  
VicUI public layout_gui {w} {
	$self instvar asm_ controlMenu_ videoAgent_ vpipe_ vframe_ exitCmd_ localChannel_
	
	set vframe_ [new VisualFrame $w.top $self]

	set asm_ [new ActiveSourceManager $self $w.top $videoAgent_ vertical $localChannel_]	
	foreach i { 1 2 3 4 } {
		bind . <Key-$i> "$asm_ redecorate $i"
	}
    
	pack $w.top -expand 1 -fill both

        set controlMenu_ [new ControlMenu $self $videoAgent_ $vpipe_ $vframe_ $asm_ [new UISrcListWindow $w $videoAgent_]]
        #XXX a bit dangerous to have this in the GUI
	bind . <t> "$controlMenu_ build_window ; $controlMenu_ invoke_transmit"
	
	$self build_menubar $w.top.bar $controlMenu_ [new VicHelpWindow .help] "$exitCmd_"
	pack $w.top.bar -fill x -side bottom

	$asm_ init_grid $w.top.f
	pack $w.top.f -expand 1 -fill both -side top
}

#
# An accessor function for a ControlMenu tkvar value.
#
VicUI public use_hw_decode {} {
	$self instvar controlMenu_
	return [$controlMenu_ use-hw]
}

#
# An accessor function for a ControlMenu tkvar value.
#
VicUI public mute_new_sources {} {
	$self instvar controlMenu_
	return [$controlMenu_ mute-new-sources]
}

#VicUI instproc cb_switcher msg {
#	$self instvar videoAgent_
#	foreach s [$videoAgent_ active_list] {
#		#
#		# look up the site by cname.  (for backward compat
#		# with old vat's, try the IP address too)
#		#
#		if { [$s addr] == $msg || [$s sdes cname] == $msg } {
#			Switcher focus $s
#			return
#		}
#	}
#}

#
# Size the "." window using values from the resource database:
# <i>minwidth</i>, <i>minheight</i>, and <i>geometry</i> (all in
# pixels).  Specify <i>geometry</i> in the form wXh or set it to empty
# string, {}, to cancel the existing user-specified geometry and revert
# to the size requested internally by its widgets.
#
VicUI instproc set-geometry {} {
	#
	# Withdraw window so that user-placement is deferred
	# until after initial geometry is computed
	#
	global mash
	if { ![info exists mash(environ)] || $mash(environ)!="mplug" } {
	    wm withdraw .
	}
	wm geometry . [$self get_option geometry]
	update idletasks
	set minwidth [winfo reqwidth .]
	set minheight [winfo reqheight .]
	#XXX
	if { $minwidth < [$self get_option minwidth] } {
		set minwidth [$self get_option minwidth]
	}
	if { $minheight < [$self get_option minheight] } {
		set minheight [$self get_option minheight]
	}
	wm minsize . $minwidth $minheight
	if { ![info exists mash(environ)] || $mash(environ)!="mplug" } {
		wm deiconify .
	}
}

#
# Every second, update rate variables.
#
VicUI instproc periodic_update { } {
	$self instvar videoAgent_ vpipe_ id_
	if [$vpipe_ running] {
		update_rate [$videoAgent_ set session_]
	}
	update idletasks
	set id_ [after 1000 "$self periodic_update"]
}

#
VicUI instproc clean_timers { } {
	$self cancel_periodic_update
}

#
# Cancels the periodic_update.
#
VicUI instproc cancel_periodic_update { } {
        $self instvar id_
        if [info exists id_] {
                after cancel $id_
                unset id_
        }
}

#
# For the video source, <i>src</i>, set the rate variables.
#
VicUI instproc set_rate_vars {src} {
	global fpshat bpshat lhat shat
	if [info exists fpshat($src)] {
		unset fpshat($src)
		unset bpshat($src)
		unset lhat($src)
		unset shat($src)
	}
	set gain [$self get_option filterGain]
	set fpshat($src) 0
	rate_variable fpshat($src) $gain
	set bpshat($src) 0
	rate_variable bpshat($src) $gain
	set lhat($src) 0
	rate_variable lhat($src) $gain
	set shat($src) 0
	rate_variable shat($src) $gain

#XXX set guys in stat window too!
}
	
#
VicUI instproc new_hostspec {} {
	$self instvar controlMenu_
	if [info exists controlMenu_] {
		$controlMenu_ new_hostspec
	}
}

#
# Update the source description by updating the element of the global
# arrays src_info(), src_nickname(), and src_name() indexed by the
# specified <i>src</i>.  If the src_name() is changed, then this is
# reflected in the UserWindows in which this src appears.
#
VicUI instproc trigger_sdes src {
	$self instvar asm_
	
	global src_info src_nickname src_name
	#
	# Figure out best presentation from available information.
	#
	set name [$src sdes name]
	set cname [$src sdes cname]
	set addr [$src addr]
	if { $name == "" } {
		if { $cname == "" } {
			set src_nickname($src) $addr
			set info $addr/[$src format_name]

		} else {
			set src_nickname($src) $cname
			set info "$addr/[$src format_name]"
		}
	} elseif [cname_redundant $name $cname] {
		set src_nickname($src) $name
		set info $addr/[$src format_name]
	} else {
		set src_nickname($src) $name
		set info $cname/[$src format_name]
	}
	set msg [$src sdes note]
	if { $msg != "" } {
		set info $msg
	}
	set src_info($src) $info
	
	# only call change_name when name really changes
	if { ![info exists src_name($src)] || "$src_name($src)" != "$name" } {
		set src_name($src) $name
		$asm_ change_name $src
	}
}

#
VicUI instproc trigger_media src {}

#
# For the source specified by <i>src</i>, update its rate variables and its source description.
#
VicUI instproc update_decoder src {
	$self set_rate_vars $src
	$self trigger_sdes $src
}

#
# Using the specified video source, <i>src</i>, as an index, update the
# global arrays bpshat(), fpshat(), shat(), lhat(), ltext(), ftext(), btext().
#
proc update_rate src {
	global ftext btext ltext fpshat bpshat lhat shat V

	set key $src
	if [string match Session/* [$src info class]] {
		set bpshat($key) [expr 8 * [$src set nb_]]
		set fpshat($key) [$src set nf_]
	} else {
		# only compute loss statistic for network side
		set p [$src layer-stat np_]
		set s [$src ns]
		set shat($key) $s
		set lhat($key) [expr $s-$p]
		if {$shat($key) <= 0.} {
			set loss 0
		} else {
			set loss [expr 100*$lhat($key)/$shat($key)]
		}
		if {$loss < .1} {
			set ltext($key) (0%)
		} elseif {$loss < 9.9} {
			set ltext($key) [format "(%.1f%%)" $loss]
		} else {
			set ltext($key) [format "(%.0f%%)" $loss]
		}
		set bpshat($key) [expr 8 * [$src layer-stat nb_]]
		set fpshat($key) [$src layer-stat nf_]
	}

	set fps $fpshat($key)
	set bps $bpshat($key)
	
	if { $fps < .1 } {
		set fps "0 f/s"
	} elseif { $fps < 10 } {
		set fps [format "%.1f f/s" $fps]
	} else {
		set fps [format "%2.0f f/s" $fps]
	}
	if { $bps < 1 } {
		set bps "0 bps"
	} elseif { $bps < 1000 } {
		set bps [format "%3.0f bps" $bps]
	} elseif { $bps < 1000000 } {
		set bps [format "%3.0f kb/s" [expr $bps / 1000]]
	} else {
		set bps [format "%.1f Mb/s" [expr $bps / 1000000]]
	}
	set ftext($key) $fps
	set btext($key) $bps
}

#
# A toggle-able dismissable toplevel window for displaying a bulleted list of tips to help the vic user.
#
Class VicHelpWindow -superclass HelpWindow

#
# Instantiate, but do not yet display or iconify, a dismissable toplevel
# window using the provided widgetpath, <i>w</i>.  Within <i>w</i>,
# create a bulleted list of tips to help the vic user.  (This method is
# called when the user toggles this window for the first time.)
#
VicHelpWindow instproc build w {

	$self create-window $w "Help" {

"Transmit video by clicking on the ``Transmit'' button \
in the ``Menu'' window.  You need video capture hardware to do this."

"Incoming video streams appear in the main vic window.  \
If you see the message ``Waiting for video...'', then no one is transmitting \
video to the conference address you're running on.  Otherwise, you'll \
see a thumbnail sized image and accompanying information for each source. \
Click on the thumbnail to open a larger viewing window.  You can tile the \
thumbnails in multiple columns using the ``Tile'' menu in the ``Menu'' window."
"Clicking on the ``mute'' button for a given source will \
turn off decoding.  It is usually a good idea to do \
this for your own, looped-back transmission."

"The transmission rate is controlled with the bit-rate \
and frame-rate sliders in the ``Transmission'' panel of the ``Menu'' window.  \
The more restrictive setting limits the transmission rate."

"The video windows need not be fixed to a given source. \
The ``Mode...'' menu attached to a viewing window allows you to specify \
voice-switched and/or timer-switched modes.   In timer-switched mode, the \
window automatically cycles through (unmuted) sources, while in \
oice-switched mode, the window switches to whomever is talking \
(using cues from vat).  You can have more than one voice-switched window, \
which results in a simple LRU allocation of the windows to most recent \
speakers.  See the man page for more details."

"If the user interface looks peculiar, you might \
have X resources that conflict with tk.  A common problem is \
defining ``*background'' and/or ``*foreground''."

"Bugs and suggestions to mash-developers@mash.cs.berkeley.edu.  Thanks."
	}
}

#
# Build the audio user-interface for the AudioAgent, <i>agent</i>.
#
VicUI instproc create-audio-ui agent {
	$self instvar audioUI_ path_
	set audioUI_ [new VicAudioUI $path_.top.f $self $agent]
}

#
# Don't use this class.  It will be obsoleted soon.
#
Class VicAudioUI

#
VicAudioUI instproc init { w mainUI agent } {
	$self instvar mainUI_ panel_
	set mainUI_ $mainUI 

	set panel_ [new AudioPanel $w $agent]

	#XXX 
	global meterDisable
	set meterDisable 0
	$panel_ install_meters

	#XXX mute mike
	$panel_ ptt-release
	$agent select_format PCM 2
}

#
VicAudioUI instproc destroy {} {
	$self instvar panel_
	delete $panel_
}

#
VicAudioUI instproc register src {
	puts "VicAudioUI::register [$src getid]"
}

#
VicAudioUI instproc activate src {
	puts "ACTIVATE: [$src getid]"
}

#
VicAudioUI instproc trigger_media src {
	puts "AUDIO-FROM: [$src getid]"
}

#
VicAudioUI instproc trigger_idle src {
	puts "IDLE: [$src getid] [$src lost]"
}

#
VicAudioUI instproc trigger_sdes src {
	puts "SDES: [$src getid]"
}

